'use strict';

exports.__esModule = true;
exports.LazyLoadWatcher = undefined;

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _remove = require('lodash/remove');

var _remove2 = _interopRequireDefault(_remove);

var _throttle = require('lodash/throttle');

var _throttle2 = _interopRequireDefault(_throttle);

var _debounce = require('lodash/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _consts = require('./consts');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import lazyLoadClassNames from './styles.module.scss';


function createImageOnloadHandler() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var _options = (0, _extends3.default)({ loadedClassName: _consts.LAZY_IMAGE_LOADED_CLASS_NAME }, options);
    return function handleImageOnload() {
        this.classList.add(_options.loadedClassName);
        // this.classList.add(lazyLoadClassNames['reactLazyLoaded']);
        this.removeEventListener('load', handleImageOnload);
    };
}

var DEFAULT_WATCHER_OPTIONS = {
    inViewportOffset: _consts.DEFAULT_IN_VIEWPORT_OFFSET,
    throttleWait: _consts.THROTTLE_WAIT_MS,
    lazyClassName: _consts.LAZY_IMAGE_CLASS_NAME,
    loadedClassName: _consts.LAZY_IMAGE_LOADED_CLASS_NAME,
    scope: document // 搜索懒加载图片的范围（父dom）
};

var LazyLoadWatcher = function () {
    function LazyLoadWatcher(scrollDom) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        (0, _classCallCheck3.default)(this, LazyLoadWatcher);
        this.scrollDom = null;
        this.lazyElementListCache = [];
        this.isWatching = false;

        this.options = (0, _extends3.default)({}, DEFAULT_WATCHER_OPTIONS, options);

        this.handleScroll = (0, _throttle2.default)(this.handleScroll.bind(this), this.options.throttleWait);
        this.setWatchImages = (0, _debounce2.default)(this.setWatchImages.bind(this), 300);
        this.handleElementLazyLoad = this.handleElementLazyLoad.bind(this);
        if (typeof scrollDom === 'string') {
            this.scrollDom = document.querySelector(scrollDom);
        } else {
            this.scrollDom = scrollDom;
        }
        if (!this.scrollDom) {
            throw new Error('LazyLoadWatcher initialize error: `scrollDom` does not exists.');
        }
    }

    LazyLoadWatcher.prototype.setWatchImages = function setWatchImages(lazyImagesDomList) {
        var _this = this;

        if (lazyImagesDomList) {
            this.lazyElementListCache = [].concat((0, _from2.default)(lazyImagesDomList));
        } else {
            this.lazyElementListCache = (0, _from2.default)(this.options.scope.querySelectorAll('.' + this.options.lazyClassName)) || [];
        }

        // 过滤出已经加载的图片
        this.lazyElementListCache = this.lazyElementListCache.filter(function (i) {
            return !i.classList.contains(_this.options.loadedClassName);
        });

        if (!this.isImageListEmpty()) {
            this.startWatch();
            this.triggerScrollManually();
        }
    };

    LazyLoadWatcher.prototype.isImageListEmpty = function isImageListEmpty() {
        return !this.lazyElementListCache || this.lazyElementListCache.length <= 0;
    };

    LazyLoadWatcher.prototype.triggerScrollManually = function triggerScrollManually() {
        // 主动触发一次scroll事件
        this.handleScroll();
        // 因为有throttle的存在，mount和update之间间隔会小于300，导致不触发，所以这里setTimeout再次执行一次
        setTimeout(this.handleScroll, this.options.throttleWait + 100);
    };

    LazyLoadWatcher.prototype.handleElementLazyLoad = function handleElementLazyLoad(element) {
        if ((0, _utils.isElementInViewport)(element, this.scrollDom, this.options.inViewportOffset)) {
            if (element.src !== element.dataset.src) {
                (0, _utils.adjustImageWidth)(this.options.scope, element);

                // 先绑定事件，防止从cache中设置src时load事件比绑定先触发
                element.addEventListener('load', createImageOnloadHandler(this.options));
                // eslint-disable-next-line no-param-reassign
                element.src = element.dataset.src;
                this.removeElementFromCache(element);
            }
        }
    };

    LazyLoadWatcher.prototype.removeElementFromCache = function removeElementFromCache(element) {
        (0, _remove2.default)(this.lazyElementListCache, function (e) {
            return e === element;
        });
    };

    LazyLoadWatcher.prototype.handleScroll = function handleScroll() {
        // 如果数组已经空了，那么停止scroll监视
        if (this.isImageListEmpty()) {
            return this.stopWatch();
        }
        this.lazyElementListCache.forEach(this.handleElementLazyLoad);
    };

    LazyLoadWatcher.prototype.startWatch = function startWatch() {
        if (this.isWatching) {
            return;
        }
        this.isWatching = true;
        this.scrollDom.addEventListener('scroll', this.handleScroll);
    };

    LazyLoadWatcher.prototype.stopWatch = function stopWatch() {
        this.isWatching = false;
        this.scrollDom.removeEventListener('scroll', this.handleScroll);
    };

    return LazyLoadWatcher;
}();

exports.LazyLoadWatcher = LazyLoadWatcher;