'use strict';

exports.__esModule = true;
exports.adjustImageWidth = exports.convertToLazyHtml = exports.isElementInViewport = exports.isElementHidden = exports.getElementPosition = undefined;

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _consts = require('./consts');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import lazyLoadClassNames from './styles.module.scss';


function getElementPosition(element) {
    var rect = element.getBoundingClientRect();

    return {
        top: rect.top + window.pageYOffset,
        left: rect.left + window.pageXOffset
    };
}

function isElementHidden(element) {
    return element.offsetParent === null;
}

var DEFAULT_OFFSET = { top: 0, right: 0, bottom: 0, left: 0 };
function isElementInViewport(element, container) {
    var customOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : DEFAULT_OFFSET;

    if (isElementHidden(element)) {
        return false;
    }

    var top = void 0;
    var bottom = void 0;
    var left = void 0;
    var right = void 0;

    if (typeof container === 'undefined' || container === window) {
        top = window.pageYOffset;
        left = window.pageXOffset;
        bottom = top + window.innerHeight;
        right = left + window.innerWidth;
    } else {
        var containerPosition = getElementPosition(container);

        top = containerPosition.top;
        left = containerPosition.left;
        bottom = top + container.offsetHeight;
        right = left + container.offsetWidth;
    }

    var elementPosition = getElementPosition(element);

    return top <= elementPosition.top + element.offsetHeight + customOffset.top && bottom >= elementPosition.top - customOffset.bottom && left <= elementPosition.left + element.offsetWidth + customOffset.left && right >= elementPosition.left - customOffset.right;
}

var DEFAULT_CONVERT_OPTIONS = {
    defaultSrc: _consts.BLANK_IMAGE_DATA_URI,
    lazyClassName: _consts.LAZY_IMAGE_CLASS_NAME
};
function convertToLazyHtml(html) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var _options = (0, _extends3.default)({}, DEFAULT_CONVERT_OPTIONS, options);

    var domParser = new DOMParser();
    var xmlSerializer = new XMLSerializer();

    var _document = domParser.parseFromString(html, 'text/html');

    if (_document && _document.querySelectorAll) {
        var _allImgs = _document.querySelectorAll('img');
        (0, _from2.default)(_allImgs).forEach(function (i) {
            // eslint-disable-next-line no-param-reassign
            i.dataset.src = i.src;
            // eslint-disable-next-line no-param-reassign
            i.src = _options.defaultSrc;
            i.classList.add(_options.lazyClassName);
            // i.classList.add(lazyLoadClassNames['reactLazyImg']);
        });
        return xmlSerializer.serializeToString(_document);
    }
    return html;
}

// 如果图片宽度超过了容器宽度，那么重新等比例计算新的高宽设置到图片上
function adjustImageWidth(container, image) {
    // this.options.scope.offsetWidth
    if (!container || !image) {
        return;
    }
    var containerWidth = container.offsetWidth;
    var imageStyleWidth = parseFloat(image.style.width);
    var imageStyleHeight = parseFloat(image.style.height);
    if (!imageStyleWidth || !imageStyleHeight) {
        return;
    }

    // 如果图片比容器宽，那么这里按比例纠正图片高度
    if (imageStyleWidth > containerWidth) {
        var newHeight = imageStyleHeight / imageStyleWidth * containerWidth;
        image.style.width = containerWidth + 'px';
        image.style.height = parseInt(newHeight, 10) + 'px';
    }
}

exports.getElementPosition = getElementPosition;
exports.isElementHidden = isElementHidden;
exports.isElementInViewport = isElementInViewport;
exports.convertToLazyHtml = convertToLazyHtml;
exports.adjustImageWidth = adjustImageWidth;